@@ -26,10 +26,10 @@
 #include "gspell-inline-checker-text-buffer.h"
 #include <string.h>
 #include <glib/gi18n-lib.h>
+#include "gspellregion.h"
 #include "gspell-checker.h"
 #include "gspell-text-buffer.h"
 #include "gspell-text-iter.h"
-#include "gspell-text-region.h"
 #include "gspell-utils.h"
 
 struct _GspellInlineCheckerTextBuffer
@@ -47,7 +47,7 @@ struct _GspellInlineCheckerTextBuffer
 
 	GtkTextMark *mark_click;
 
-	GspellTextRegion *scan_region;
+	GspellRegion *scan_region;
 	guint timeout_id;
 
 	/* When a word is being typed, it should not be marked as misspelled,
@@ -87,6 +87,58 @@ typedef enum
 
 G_DEFINE_TYPE (GspellInlineCheckerTextBuffer, _gspell_inline_checker_text_buffer, G_TYPE_OBJECT)
 
+/* FIXME properly. Workaround for bug in GtkTextView:
+ * https://bugzilla.gnome.org/show_bug.cgi?id=763741
+ */
+static void
+queue_draw_hack (GspellInlineCheckerTextBuffer *spell)
+{
+	GSList *l;
+
+	for (l = spell->views; l != NULL; l = l->next)
+	{
+		GtkWidget *view = GTK_WIDGET (l->data);
+		gtk_widget_queue_draw (view);
+	}
+}
+
+/* Remove the highlight_tag only if present. If gtk_text_buffer_remove_tag() is
+ * called when the tag is not present, GtkTextView anyway queues a redraw, which
+ * we want to avoid (it can lead to an infinite loop).
+ */
+static void
+remove_highlight_tag_if_present (GspellInlineCheckerTextBuffer *spell,
+				 const GtkTextIter             *start,
+				 const GtkTextIter             *end)
+{
+	gboolean remove = FALSE;
+
+	if (gtk_text_iter_has_tag (start, spell->highlight_tag))
+	{
+		remove = TRUE;
+	}
+	else
+	{
+		GtkTextIter iter = *start;
+
+		if (gtk_text_iter_forward_to_tag_toggle (&iter, spell->highlight_tag) &&
+		    gtk_text_iter_compare (&iter, end) < 0)
+		{
+			remove = TRUE;
+		}
+	}
+
+	if (remove)
+	{
+		gtk_text_buffer_remove_tag (spell->buffer,
+					    spell->highlight_tag,
+					    start,
+					    end);
+
+		queue_draw_hack (spell);
+	}
+}
+
 static void
 check_word (GspellInlineCheckerTextBuffer *spell,
 	    const GtkTextIter             *start,
@@ -258,47 +310,6 @@ get_visible_region (GtkTextView *view,
 	gtk_text_iter_forward_line (end);
 }
 
-/* A TextRegion can contain empty subregions. So checking the number of
- * subregions is not sufficient.
- * When calling text_region_add() with equal iters, the subregion is not
- * added. But when a subregion becomes empty, due to text deletion, the
- * subregion is not removed from the TextRegion.
- */
-static gboolean
-is_text_region_empty (GspellTextRegion *region)
-{
-	GspellTextRegionIterator region_iter;
-
-	if (region == NULL)
-	{
-		return TRUE;
-	}
-
-	_gspell_text_region_get_iterator (region, &region_iter, 0);
-
-	while (!_gspell_text_region_iterator_is_end (&region_iter))
-	{
-		GtkTextIter region_start;
-		GtkTextIter region_end;
-
-		if (!_gspell_text_region_iterator_get_subregion (&region_iter,
-								 &region_start,
-								 &region_end))
-		{
-			return TRUE;
-		}
-
-		if (!gtk_text_iter_equal (&region_start, &region_end))
-		{
-			return FALSE;
-		}
-
-		_gspell_text_region_iterator_next (&region_iter);
-	}
-
-	return TRUE;
-}
-
 /* Returns TRUE if there is a current word. */
 static gboolean
 get_current_word_boundaries (GtkTextBuffer *buffer,
@@ -334,8 +345,8 @@ check_visible_region_in_view (GspellInlineCheckerTextBuffer *spell,
 {
 	GtkTextIter visible_start;
 	GtkTextIter visible_end;
-	GspellTextRegion *intersect;
-	GspellTextRegionIterator intersect_iter;
+	GspellRegion *intersect;
+	GspellRegionIter intersect_iter;
 
 	if (spell->scan_region == NULL)
 	{
@@ -352,12 +363,13 @@ check_visible_region_in_view (GspellInlineCheckerTextBuffer *spell,
 		gtk_text_buffer_get_bounds (spell->buffer, &visible_start, &visible_end);
 	}
 
-	intersect = _gspell_text_region_intersect (spell->scan_region,
-						   &visible_start,
-						   &visible_end);
+	intersect = _gspell_region_intersect (spell->scan_region,
+					      &visible_start,
+					      &visible_end);
 
-	if (intersect == NULL)
+	if (_gspell_region_is_empty (intersect))
 	{
+		g_clear_object (&intersect);
 		return;
 	}
 
@@ -370,42 +382,39 @@ check_visible_region_in_view (GspellInlineCheckerTextBuffer *spell,
 						 &current_word_start,
 						 &current_word_end))
 		{
-			gtk_text_buffer_remove_tag (spell->buffer,
-						    spell->highlight_tag,
-						    &current_word_start,
-						    &current_word_end);
+			remove_highlight_tag_if_present (spell,
+							 &current_word_start,
+							 &current_word_end);
 
-			_gspell_text_region_subtract (intersect,
-						      &current_word_start,
-						      &current_word_end);
+			_gspell_region_subtract (intersect,
+						 &current_word_start,
+						 &current_word_end);
 
 			/* Be sure that the current word will be re-checked
 			 * later when it will no longer be the current word.
 			 */
-			_gspell_text_region_add (spell->scan_region,
-						 &current_word_start,
-						 &current_word_end);
+			_gspell_region_add (spell->scan_region,
+					    &current_word_start,
+					    &current_word_end);
+
+			if (_gspell_region_is_empty (intersect))
+			{
+				g_clear_object (&intersect);
+				return;
+			}
 		}
 	}
 
-	if (is_text_region_empty (intersect))
-	{
-		_gspell_text_region_destroy (intersect);
-		goto out;
-	}
+	_gspell_region_get_start_region_iter (intersect, &intersect_iter);
 
-	_gspell_text_region_get_iterator (intersect, &intersect_iter, 0);
-
-	while (!_gspell_text_region_iterator_is_end (&intersect_iter))
+	while (!_gspell_region_iter_is_end (&intersect_iter))
 	{
 		GtkTextIter start;
 		GtkTextIter end;
 		GtkTextIter orig_start;
 		GtkTextIter orig_end;
 
-		if (!_gspell_text_region_iterator_get_subregion (&intersect_iter,
-								 &start,
-								 &end))
+		if (!_gspell_region_iter_get_subregion (&intersect_iter, &start, &end))
 		{
 			break;
 		}
@@ -422,27 +431,19 @@ check_visible_region_in_view (GspellInlineCheckerTextBuffer *spell,
 		g_assert (gtk_text_iter_compare (&start, &orig_start) <= 0);
 		g_assert (gtk_text_iter_compare (&orig_end, &end) <= 0);
 
-		_gspell_text_region_subtract (spell->scan_region, &start, &end);
+		_gspell_region_subtract (spell->scan_region, &start, &end);
 
-		_gspell_text_region_iterator_next (&intersect_iter);
+		_gspell_region_iter_next (&intersect_iter);
 	}
 
-	_gspell_text_region_destroy (intersect);
+	g_clear_object (&intersect);
 
-	if (is_text_region_empty (spell->scan_region))
+	if (_gspell_region_is_empty (spell->scan_region))
 	{
-		_gspell_text_region_destroy (spell->scan_region);
-		spell->scan_region = NULL;
+		g_clear_object (&spell->scan_region);
 	}
 
-out:
-	if (view != NULL)
-	{
-		/* FIXME properly. Workaround for bug in GtkTextView:
-		 * https://bugzilla.gnome.org/show_bug.cgi?id=612772
-		 */
-		gtk_widget_queue_draw (GTK_WIDGET (view));
-	}
+	queue_draw_hack (spell);
 }
 
 static void
@@ -506,10 +507,10 @@ add_subregion_to_scan (GspellInlineCheckerTextBuffer *spell,
 {
 	if (spell->scan_region == NULL)
 	{
-		spell->scan_region = _gspell_text_region_new (spell->buffer);
+		spell->scan_region = _gspell_region_new (spell->buffer);
 	}
 
-	_gspell_text_region_add (spell->scan_region, start, end);
+	_gspell_region_add (spell->scan_region, start, end);
 }
 
 static void
@@ -1005,6 +1006,8 @@ remove_tag_to_word (GspellInlineCheckerTextBuffer *spell,
 
 		iter = match_end;
 	}
+
+	queue_draw_hack (spell);
 }
 
 static void
@@ -1364,18 +1367,13 @@ _gspell_inline_checker_text_buffer_dispose (GObject *object)
 
 	g_clear_object (&spell->highlight_tag);
 	g_clear_object (&spell->no_spell_check_tag);
+	g_clear_object (&spell->scan_region);
 
 	g_slist_free (spell->views);
 	spell->views = NULL;
 
 	spell->mark_click = NULL;
 
-	if (spell->scan_region != NULL)
-	{
-		_gspell_text_region_destroy (spell->scan_region);
-		spell->scan_region = NULL;
-	}
-
 	if (spell->timeout_id != 0)
 	{
 		g_source_remove (spell->timeout_id);
