@@ -27,6 +27,8 @@
 
 #include <gtk/gtk.h>
 
+G_BEGIN_DECLS
+
 #define GSPELL_TYPE_INLINE_CHECKER_TEXT_BUFFER (_gspell_inline_checker_text_buffer_get_type ())
 
 G_GNUC_INTERNAL
@@ -56,6 +58,8 @@ G_GNUC_INTERNAL
 GtkTextTag *
 	_gspell_inline_checker_text_buffer_get_highlight_tag	(GspellInlineCheckerTextBuffer *spell);
 
+G_END_DECLS
+
 #endif  /* __GSPELL_INLINE_CHECKER_TEXT_BUFFER_H__ */
 
 /* ex:set ts=8 noet: */
